@@ -24,6 +24,7 @@ class ProtocolCode(object):
     IS_CONTROLLER_CONNECTED = 0x14
     READ_NEXT_ERROR = 0x15
     SET_FRESH_MODE = 0x16
+    GET_FRESH_MODE = 0x17   
     SET_FREE_MODE = 0x1A
     IS_FREE_MODE = 0x1B
 
@@ -40,6 +41,8 @@ class ProtocolCode(object):
     STOP = 0x29
     IS_IN_POSITION = 0x2A
     IS_MOVING = 0x2B
+    GET_ANGLE = 0x2C
+    GET_COORD = 0x2D
 
     # JOG MODE AND OPERATION
     JOG_ANGLE = 0x30
@@ -88,6 +91,8 @@ class ProtocolCode(object):
     SET_COLOR = 0x6A
     SET_ELETRIC_GRIPPER = 0x6B
     INIT_ELETRIC_GRIPPER = 0x6C
+    
+    GET_ACCEI_DATA = 0x73
 
     # Basic
     SET_BASIC_OUTPUT = 0xA0
@@ -139,6 +144,17 @@ class ProtocolCode(object):
     GET_SERVO_STATUS = 0xE4
     GET_SERVO_TEMPS = 0xE5
     
+    GET_BASE_COORDS = 0xF0
+    BASE_TO_SINGLE_COORDS = 0xF1
+    COLLISION = 0xF2
+    GET_BASE_COORD = 0xF3
+    GET_ALL_BASE_COORDS = 0xF4
+    WRITE_BASE_COORD = 0xF5
+    WRITE_BASE_COORDS = 0xF6
+    JOG_INC_COORD = 0xF7
+    COLLISION_SWITCH = 0xF8
+    IS_COLLISION_ON = 0xF9
+    
     # IIC
     # SET_IIC_STATE = 0xA4
     # GET_IIS_BYTE = 0xA5
@@ -210,7 +226,7 @@ class DataProcessor(object):
     def _is_frame_header(self, data, pos1, pos2):
         return data[pos1] == ProtocolCode.HEADER and data[pos2] == ProtocolCode.HEADER
 
-    def _process_received(self, data, genre):
+    def _process_received(self, data, genre, arm = 6):
         if not data:
             return []
         if genre == 177:
@@ -224,7 +240,10 @@ class DataProcessor(object):
         header_i, header_j = 0, 1
         while header_j < data_len-4:
             if self._is_frame_header(data, header_i, header_j):
-                cmd_id = data[header_i + 3]
+                if arm == 6:
+                    cmd_id = data[header_i + 3]
+                elif arm == 12:
+                    cmd_id = data[header_i + 4]
                 # compare send header and received header
                 if cmd_id == genre:
                     break
@@ -232,7 +251,10 @@ class DataProcessor(object):
             header_j += 1
         else:
             return []
-        data_len = data[header_i + 2] - 2
+        if arm == 6:
+            data_len = data[header_i + 2] - 2
+        elif arm == 12:
+            data_len = data[header_i + 3] - 2
         unique_data = [ProtocolCode.GET_BASIC_INPUT,
                        ProtocolCode.GET_DIGITAL_INPUT]
 
@@ -240,12 +262,16 @@ class DataProcessor(object):
             data_pos = header_i + 5
             data_len -= 1
         else:
-            data_pos = header_i + 4
+            if arm == 6:
+                data_pos = header_i + 4
+            elif arm == 12:
+                data_pos = header_i + 5
+                
         valid_data = data[data_pos: data_pos + data_len]
 
         # process valid data
         res = []
-        if data_len == 12 or data_len == 8 or data_len == 24:
+        if data_len in [6, 8, 12 ,24]:
             for header_i in range(0, len(valid_data), 2):
                 one = valid_data[header_i: header_i + 2]
                 res.append(self._decode_int16(one))
@@ -264,6 +290,9 @@ class DataProcessor(object):
                     res.append(0xff & data1 if data1 <0 else data1)
                 return res
             res.append(self._decode_int8(valid_data))
+        if genre == ProtocolCode.GET_ACCEI_DATA:
+            for i in range(len(res)):
+                res[i] /= 1000
         return res
 
     def _process_single(self, data):
@@ -277,7 +306,7 @@ def write(self, command, method=None):
     if method == "socket":
         data = b""
         if self.rasp:
-            self.sock.sendall(str(command).encode())
+            self.sock.send(str(command).encode())
         else:
             self.sock.sendall(bytes(command))
 
@@ -293,13 +322,13 @@ def write(self, command, method=None):
                     break
         else:
             try:
-                self.sock.settimeout(0.1)
+                self.sock.settimeout(1)
                 data = self.sock.recv(1024)
             except:
                 data = b''
         return data
     else:
-        self.log.debug("_write: {}".format(command))
+        self.log.debug("_write: {}".format([hex(i) for i in command]))
         self._serial_port.write(command)
         self._serial_port.flush()
 
@@ -309,14 +338,17 @@ def read(self, genre):
     k = 0
     pre = 0
     t = time.time()
-    if genre == 0xB1:
+    wait_time = 0.1
+    if genre == ProtocolCode.GET_SSID_PWD:
         time.sleep(0.1)
         if self._serial_port.inWaiting()>0:
             datas = self._serial_port.read(self._serial_port.inWaiting())
-            print(datas)
         return datas
-    while True and time.time() - t < 0.1: 
+    elif genre == ProtocolCode.GET_ACCEI_DATA:
+        wait_time = 1
+    while True and time.time() - t < wait_time: 
         data = self._serial_port.read()
+        # print("1:",data)
         k+=1
         if data_len == 1 and data == b'\xfa':
             datas += data
